/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class ArrayStringIterator
  extends    AbstractStringIterator
  implements IStringIterator
{
  protected String[] values = StringIterator.defaultValues;
  protected int index = 0;
  
  public ArrayStringIterator()
  {
  }
  
  public ArrayStringIterator(String[] values)
  {
    if(values != null) this.values = values;
    else               this.values = new String[0];
  }
  
  public ArrayStringIterator(ArrayStringIterator r)
  {
    if(r != null) 
    {
      index  = r.index;
      values = new String[r.values.length];
      
      System.arraycopy(r.values, 0, values, 0, values.length);
    }
    else values = new String[0];
  }

  public void addString(String n)
  {
    String[] newValues = new String[values.length+1];
    System.arraycopy(values, 0, newValues, 0, values.length);
    newValues[values.length] = n;
    values = newValues;
  }
 
  public Object clone()
  {
    return new ArrayStringIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof ArrayStringIterator))
      return false;

    ArrayStringIterator obj = (ArrayStringIterator)o;

    if(index != obj.index) return false;
    if(values.length != obj.values.length) return false;
    
    for(int i = 0; i < values.length; i++)
      if(   values[i] == null && obj.values[i] != null
         || values[i] != null && !values[i].equals(obj.values[i])
        ) return false;
    
    return true;   
  }
  
  public String value()
  {
    return values[index];
  }
  
  public boolean stop()
  {
    return index >= values.length;
  }
  
  public void shift()
  {
    index++;
  }

  public int hashCode()
  {
    int r = 113*index;
    for(int i = 0; i < values.length; i++)
    { 
      r *= 3;
      r += (values[i] == null)?0:values[i].hashCode();   
    }
    
    return r;
  }
}
