/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class NeighbourhoodFloatIterator
  extends    AbstractFloatIterator
  implements IFloatIterator
{
  protected float point = 0;
  protected int offset = 0;
  protected int index  = 0;
  
  public NeighbourhoodFloatIterator(float point, int offset)
  {
    if(offset < 0) offset = 0;
    if(offset > Integer.MAX_VALUE/2) offset = Integer.MAX_VALUE/2;
     
    this.point  = point;
    this.offset = offset;
  }

  public NeighbourhoodFloatIterator(float point)
  {
    this(point, 0);
  }
  
  public NeighbourhoodFloatIterator(NeighbourhoodFloatIterator r)
  {
    this.point  = r.point;
    this.offset = r.offset;
    this.index  = r.index;
  }

  public Object clone()
  {
    return new NeighbourhoodFloatIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof NeighbourhoodFloatIterator))
      return false;

    NeighbourhoodFloatIterator obj = (NeighbourhoodFloatIterator)o;

    return    index  == obj.index 
           && (point  == obj.point || Float.isNaN(point) && Float.isNaN(obj.point))
           && offset == obj.offset;
  }
  
  public float value()
  {
    if(Float.isNaN(point))
      switch(index)
      {
        case 0:  return Float.NaN;
        case 1:  return Float.POSITIVE_INFINITY;
        case 2:  return Float.NEGATIVE_INFINITY;
        default: return ((index&1) == 0)?
                          FloatIterator.shiftFloat(Float.NEGATIVE_INFINITY, index/2-1)
                        : FloatIterator.shiftFloat(Float.POSITIVE_INFINITY, -index/2);
      }
    else return FloatIterator.shiftFloat(point, (index%2 == 0)?(index/2):(-(index/2+1)));
  }
  
  public boolean stop()
  {
    return index > 2*offset;
  }
  
  public void shift()
  {
    index++;
    
    if(index == Integer.MAX_VALUE || Float.isNaN(point)) return;
    else if(point == Float.POSITIVE_INFINITY)
    {
      if((index&1) == 0 && index > 2) { index++; return; }
      else                                       return;
    }
    else if(point == Float.NEGATIVE_INFINITY)
    {
      if((index&1) != 0 && index > 1) { index++; return; } 
      else                                       return;
    }
    else
    {
      if(      index%2 == 0
            && point > FloatIterator.shiftFloat(Float.MAX_VALUE, - index/2)
         ||    index%2 == 1
            && point < FloatIterator.shiftFloat(-Float.MAX_VALUE, (index+1)/2)
        )
        index++;
    }
  }

  public int hashCode ()
  {
    return index + 3*Float.floatToIntBits(point) + 11*offset;
  }
}
