/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class SegmentByteIterator
  extends AbstractByteIterator
  implements IByteIterator
{
  protected byte start = 0;
  protected byte end = 0;
  protected int offset = 0;
  protected int divisor = 1;
  protected int index = 0;
  protected int subindex = 0;
  
  public SegmentByteIterator(byte start, byte end, int offset, int divisor)
  {
    if(divisor <= 0) divisor = 1;
    if(offset < 0) offset = 0;
    if(offset > Byte.MAX_VALUE) offset = Byte.MAX_VALUE;

    if((2*offset+1)*divisor >= Math.abs(end - start))
    {
      divisor = Math.abs(end - start);
      offset = 0;
    }
    
    this.start   = start;
    this.end     = end;
    this.offset  = offset;
    this.divisor = divisor;
  }

  public SegmentByteIterator(byte start, byte end, int offset)
  {
    this(start, end, offset, 1);
  }
  
  public SegmentByteIterator(byte start, byte end)
  {
    this(start, end, 0, 1);
  }
  
  public SegmentByteIterator(SegmentByteIterator r)
  {
    this.start   = r.start;
    this.end     = r.end;
    this.offset  = r.offset;
    this.divisor = r.divisor;
    this.index    = r.index;
    this.subindex = r.subindex;
  }

  public Object clone()
  {
    return new SegmentByteIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof SegmentByteIterator))
      return false;

    SegmentByteIterator obj = (SegmentByteIterator)o;

    return    index    == obj.index 
           && subindex == obj.subindex 
           && start    == obj.start 
           && end      == obj.end
           && offset   == obj.offset
           && divisor  == obj.divisor;
  }
  
  public byte value()
  {
    if(index == 0)
      return (byte)(start + subindex*((start <= end)?(1):(-1)));
    else if(index == divisor)
      return (byte)(end - subindex*((start <= end)?(1):(-1)));
    else
      return (byte)(start + (end-start)*index/divisor + ((subindex%2 == 0)?(subindex/2):(-(subindex+1)/2)));
  }
  
  public boolean stop()
  {
    return index > divisor;
  }
  
  public void shift()
  {
    subindex++;

    if(   index == 0
       || index == divisor
      )
    {
      if(subindex > offset)
      {
        index++;
        subindex = 0;
      }
    }
    else if(subindex > 2*offset)
    {
      index++;
      subindex = 0;
    }
  }
  
  public int hashCode ()
  {
    return index + 3*subindex + 5*start + 7*end + 11*offset + 13*divisor;
  }
}
