/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang.synchronization;

import jatva.tracer.messages.SimpleUserInfoMessage;
import jatva.tracer.messages.TaggedUserInfoMessage;
import jatva.tracer.messages.TracerMessage;
import jatva.util.IdentityWrapper;

import java.util.LinkedList;

/**
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public class TraceMessagesBuffer
{
    public TraceMessagesBuffer()
    { buffer = new LinkedList<IdentityWrapper<TracerMessage>>(); }

    private LinkedList<IdentityWrapper<TracerMessage>> buffer;

    public LinkedList<IdentityWrapper<TracerMessage>> getBuffer()
    { return buffer; }

    private boolean suppressUserInfo = false;

    public void setSuppressUserInfo( boolean suppressUserInfo )
    { this.suppressUserInfo = suppressUserInfo; }

    private boolean traceImmediately = true;

    private void _add( TracerMessage tm, boolean is_new )
    {
        if(    suppressUserInfo
            && (    tm instanceof SimpleUserInfoMessage
                 || tm instanceof TaggedUserInfoMessage
               )
          )
        {
            return;
        }

        if( is_new )
            tm.makeStringForm();

        if( traceImmediately )
            tm.trace();
        else
            buffer.addLast( new IdentityWrapper<TracerMessage>( tm ) );
    }

    public void addNew( TracerMessage tm )
    {
        _add( tm, true );
    }

    public void addOld( TracerMessage tm )
    {
        _add( tm, false );
    }

    public void addAll( LinkedList<TracerMessage> list )
    {
        for( TracerMessage tm : list )
        {
            buffer.addLast( new IdentityWrapper<TracerMessage>( tm ) );
        }
    }

    public void switchToBufferredTracing()
    {
        assert buffer.size() == 0;
        traceImmediately = false;
    }

    public void switchToImmediateTracing()
    {
        for( IdentityWrapper<TracerMessage> iw : buffer )
        {
            iw.value().trace();
        }
        buffer.clear();
        traceImmediately = true;
    }

    public void clear()
    { buffer.clear(); }

    public boolean isEmpty()
    { return buffer.isEmpty(); }

    public synchronized LinkedList<TracerMessage> extractBufferedMessages()
    {
        LinkedList<TracerMessage> res = new LinkedList<TracerMessage>();

        for( IdentityWrapper<TracerMessage> iw : buffer )
        {
            res.addLast( iw.value() );
        }

        buffer.clear();
        return res;
    }
}
